//--------------------------------------------------------------------------------------
// Compute and filter/smooth gradients from a 3D voxel volume texture
// using compute shaders
//
// DirectX/HLSL implementation by
// Christian B. Mendl, 2012
//--------------------------------------------------------------------------------------

#ifndef GRADIENT_FILTER_H
#define GRADIENT_FILTER_H

#include <d3d11.h>



class GradientFilter
{

public:

	GradientFilter();

	HRESULT CreateResources(ID3D11Device *pd3dDevice, UINT iWidth, UINT iHeight, UINT iDepth, wchar_t *fileName);

	// Process the volume voxel data: compute and filter gradients
	void ProcessVolume(ID3D11DeviceContext *pImmediateContext);

	void GetTextureRV(ID3D11ShaderResourceView **ppTexRV);

	void Cleanup();

private:
	
	// Texture dimensions
	UINT								m_iWidth;
	UINT								m_iHeight;
	UINT								m_iDepth;

	// Compute shaders
	ID3D11ComputeShader*				m_pCSGradient;	// gradient calculation
	ID3D11ComputeShader*				m_pCSFilter;	// gradient filter

	// Voxel volume texture
	ID3D11Texture3D *m_pTexVolume;
	ID3D11ShaderResourceView *m_pTexVolumeRV;

	// Intermediate volume gradient and voxel texture (before filtering)
	ID3D11Texture3D*					m_pTexGradientVoxel;		// gradient in xyz, voxel intensity in w (alpha)
	ID3D11ShaderResourceView*			m_pTexGradientVoxelRV;
	ID3D11UnorderedAccessView*			m_pTexGradientVoxelUAV;

	// Final volume gradient and voxel texture
	ID3D11Texture3D*					m_pTexFilteredGradientVoxel;		// gradient in xyz, voxel intensity in w (alpha)
	ID3D11ShaderResourceView*			m_pTexFilteredGradientVoxelRV;
	ID3D11UnorderedAccessView*			m_pTexFilteredGradientVoxelUAV;
};



#endif
