//--------------------------------------------------------------------------------------
// Render front- and back-face model positions to texture
//
// DirectX/HLSL implementation by
// Christian B. Mendl, 2012
//
// Reference:
// http://graphicsrunner.blogspot.de/2009/01/volume-rendering-102-transfer-functions.html
//--------------------------------------------------------------------------------------

#ifndef MODEL_POSITION_H
#define MODEL_POSITION_H

#include <d3d11.h>
#include <xnamath.h>


class ModelPosition
{

public:

	ModelPosition();

	HRESULT CreateResources(ID3D11Device *pd3dDevice, UINT iWindowWidth, UINT iWindowHeight);

	void Render(ID3D11DeviceContext *pImmediateContext, const XMMATRIX *pWVP);

	void GetTextureRV(ID3D11ShaderResourceView *ppTexRV[2]);

	void Cleanup();

private:

	// Render front- and back-face positions to texture
	ID3D11VertexShader*					m_pVSModelPosition;
	ID3D11InputLayout*					m_pVLModelPosition;
	ID3D11PixelShader*					m_pPSModelPosition;

	// Position textures for front and back side
	ID3D11Texture2D*					m_pTexPosition[2];
	ID3D11ShaderResourceView*			m_pTexPositionRV[2];
	ID3D11RenderTargetView*				m_pTexPositionRTV[2];

	// Rasterizer states
	ID3D11RasterizerState*				m_pRasterizerStateCullBack;
	ID3D11RasterizerState*				m_pRasterizerStateCullFront;

	// Constant buffer
	ID3D11Buffer*						m_pCBEveryFrame;
};



#endif
