//--------------------------------------------------------------------------------------
// Ray-casting main rendering pass
//
// DirectX/HLSL implementation by
// Christian B. Mendl, 2012
//
// Reference:
// http://graphicsrunner.blogspot.de/2009/01/volume-rendering-102-transfer-functions.html
//--------------------------------------------------------------------------------------

#ifndef RAY_CAST_H
#define RAY_CAST_H

#include <d3d11.h>
#include <xnamath.h>


class RayCast
{

public:

	RayCast();

	HRESULT CreateResources(ID3D11Device *pd3dDevice, UINT iWindowWidth, UINT iWindowHeight);

	void Render(ID3D11DeviceContext *pImmediateContext, ID3D11ShaderResourceView *pTexGradientVoxelRV, ID3D11ShaderResourceView *pTexPositionRV[2], const XMMATRIX *pWVP, const XMVECTOR *vLightPos);

	void Cleanup();

private:

	HRESULT CreateTransferTexture(ID3D11Device *pd3dDevice);

	// Vertex and pixel shaders of the ray-casting algorithm
	ID3D11VertexShader*					m_pVSRayCast;
	ID3D11InputLayout*					m_pVLRayCast;
	ID3D11PixelShader*					m_pPSRayCast;

	// Transfer texture
	ID3D11Texture1D*					m_pTexTransfer;
	ID3D11ShaderResourceView*			m_pTexTransferRV;

	// Constant buffers
	ID3D11Buffer*						m_pCBEveryFrameVS;
	ID3D11Buffer*						m_pCBImmutablePS;
	ID3D11Buffer*						m_pCBEveryFramePS;
};



#endif
