//--------------------------------------------------------------------------------------
// Filter gradients in compute shader
//
// DirectX/HLSL implementation by
// Christian B. Mendl, 2012
//
// Reference:
// http://graphicsrunner.blogspot.de/2009/01/volume-rendering-102-transfer-functions.html
//--------------------------------------------------------------------------------------


#pragma pack_matrix(row_major)


//--------------------------------------------------------------------------------------
// Textures
//--------------------------------------------------------------------------------------

Texture3D<float4> txInput : register(t0);

// Output result UAV (unordered access view)
RWTexture3D<float4> txOutput : register(u0);

// Gaussian filter weights obtained from Exp[-(x^2 + y^2 + z^2)/2] for x, y, z = -1, 0, 1 and subsequent normalization
static const float weights[3][3][3] = {
	{ { 0.0205863, 0.033941,  0.0205863 }, { 0.033941,  0.0559593, 0.033941  }, { 0.0205863, 0.033941, 0.0205863 } },
	{ { 0.033941,  0.0559593, 0.033941  }, { 0.0559593, 0.0922613, 0.0559593 }, { 0.033941, 0.0559593, 0.033941  } },
	{ { 0.0205863, 0.033941,  0.0205863 }, { 0.033941,  0.0559593, 0.033941  }, { 0.0205863, 0.033941, 0.0205863 } } };


//--------------------------------------------------------------------------------------
// Filter compute shader
//--------------------------------------------------------------------------------------

[numthreads(8, 8, 8)]
void Filter(uint3 threadID : SV_DispatchThreadID)
{
	float3 avrgrad = float3(0, 0, 0);

	for (uint i = 0; i < 3; i++) {
		for (uint j = 0; j < 3; j++) {
			for (uint k = 0; k < 3; k++) {
				avrgrad += weights[i][j][k] * txInput[threadID + uint3(i, j, k) - uint3(1, 1, 1)].xyz;
			}
		}
	}

	// Pass along w-value
	txOutput[threadID] = float4(avrgrad, txInput[threadID].w);
}
