//--------------------------------------------------------------------------------------
// Calculate gradient in compute shader
//
// DirectX/HLSL implementation by
// Christian B. Mendl, 2012
//
// Reference:
// http://graphicsrunner.blogspot.de/2009/01/volume-rendering-102-transfer-functions.html
//--------------------------------------------------------------------------------------


#pragma pack_matrix(row_major)


//--------------------------------------------------------------------------------------
// Textures and samplers
//--------------------------------------------------------------------------------------

Texture3D<float> txVolume : register(t0);

// Output result UAV (unordered access view)
RWTexture3D<float4> txGradient : register(u0);


//--------------------------------------------------------------------------------------
// Gradient compute shader
//--------------------------------------------------------------------------------------

[numthreads(8, 8, 8)]
void ComputeGradient(uint3 threadID : SV_DispatchThreadID)
{
	// use long differences
	float3 vdiff = -float3(
		txVolume[threadID + uint3(1,0,0)] - txVolume[threadID - uint3(1,0,0)],
		txVolume[threadID + uint3(0,1,0)] - txVolume[threadID - uint3(0,1,0)],
		txVolume[threadID + uint3(0,0,1)] - txVolume[threadID - uint3(0,0,1)]);
	float lg = length(vdiff);
	vdiff = (lg == 0 ? float3(0, 0, 0) : vdiff/lg);

	txGradient[threadID] = float4(vdiff, txVolume[threadID]);
}
