//--------------------------------------------------------------------------------------
// DirectX/HLSL implementation by
// Christian B. Mendl, 2012
//
// Reference:
// http://graphicsrunner.blogspot.de/2009/01/volume-rendering-102-transfer-functions.html
//--------------------------------------------------------------------------------------

#include "util.h"
#include <d3dx11.h>
#include <d3dcompiler.h>
#include <stdio.h>
#include <io.h>
#include <fcntl.h>


//--------------------------------------------------------------------------------------
// Helper for compiling shaders with D3DX11
//--------------------------------------------------------------------------------------
HRESULT CompileShaderFromFile(WCHAR *szFileName, LPCSTR szEntryPoint, LPCSTR szShaderModel, ID3DBlob **ppBlobOut)
{
	HRESULT hr;

	DWORD dwShaderFlags = D3DCOMPILE_ENABLE_STRICTNESS;
	#if defined(DEBUG) || defined(_DEBUG)
	// Set the D3DCOMPILE_DEBUG flag to embed debug information in the shaders.
	// Setting this flag improves the shader debugging experience, but still allows 
	// the shaders to be optimized and to run exactly the way they will run in 
	// the release configuration of this program.
	dwShaderFlags |= D3DCOMPILE_DEBUG;
	#endif

	ID3DBlob *pErrorBlob;
	hr = D3DX11CompileFromFile(szFileName, NULL, NULL, szEntryPoint, szShaderModel, 
		dwShaderFlags, 0, NULL, ppBlobOut, &pErrorBlob, NULL);
	if(FAILED(hr))
	{
		if(pErrorBlob != NULL)
			OutputDebugStringA((char *)pErrorBlob->GetBufferPointer());
		if(pErrorBlob) pErrorBlob->Release();
		return hr;
	}
	if(pErrorBlob) pErrorBlob->Release();

	return S_OK;
}


//--------------------------------------------------------------------------------------
// Round up to the next highest power of 2
//--------------------------------------------------------------------------------------
UINT NextPow2(UINT v)
{
	// see 'Bit Twiddling Hacks'
	// http://graphics.stanford.edu/~seander/bithacks.html

	v--;

	// Copy the highest set bit to all of the lower bits
	v |= v >> 1;
	v |= v >> 2;
	v |= v >> 4;
	v |= v >> 8;
	v |= v >> 16;
	// Add one, e.g., 00011111_2 + 1 = 00100000_2
	v++;

	return v;
}


//----------------------------------------------------------------------------------------------------
// Allocate console window and redirect 'printf'
//----------------------------------------------------------------------------------------------------
void SetupConsole()
{
	AllocConsole();
	HANDLE hCrt = GetStdHandle(STD_OUTPUT_HANDLE);

	// Redirect unbuffered STDOUT to the console
	long lStdHandle = (long)GetStdHandle(STD_OUTPUT_HANDLE);
	int hConHandle = _open_osfhandle(lStdHandle, _O_TEXT);
	FILE *hf = _fdopen(hConHandle, "w");
	*stdout = *hf;
	setvbuf(stdout, NULL, _IONBF, 0);
}
